<!-- extend from base layout -->
{% extends "base.html" %}

{% block content %}
<article class="related">
<h1>{{keyword}}</h1>
<p>{% for keyword in related %}{% if loop.index>1 %}, {% endif %}<a href="/rel/keyword/{{keyword|urlencode}}">{{keyword}}</a>{% endfor %}</p>
{% for entry in entries %}
<section>
<h2> ⇒ <a href="{{entry['uri']|urlencode}}">{{entry['title']|escape}}</a></h2>
<p>{% for keyword in entry['keywords'] %}{% if loop.index>1 %}, {% endif %}<a href="/rel/keyword/{{keyword|urlencode}}">{{keyword}}</a>{% endfor %}</p>
</section>
{% endfor %}
</article>
{% endblock %}
