body {
}
#user-nav {
   position: fixed;
   top: 0px;
   right: 1em;
}

body > header {
   position: fixed;
   top: 0px;
   left: 0px;
   padding-left: 1em;
   width: 100%;
   border-bottom: 1px solid rgb(200,200,200);
   background-color: rgb(255,255,255);
   z-index: 100;
}
body > header > h1 {
   margin-bottom: -0.1825em;
}
body > main {
   clear: both;
   margin-top: 4em;
}

.uk-form-horizontal .uk-form-label {
   width: auto;
   font-size: 1em;
}

.uk-form-horizontal .uk-form-controls {
   margin-left: 10em;
}

#editor-form-content-create .uk-form-controls {
   margin-left: 5em;
}

#editor-form-content-create .uk-select {
   width: 10em;
}

#editor-form-content-create .uk-input[name=type] {
   width: 30em;
}

.editor-content-item-menu {
   float: left;
   padding-left: 0.25em;
   width: 1.5em;
}

.editor-content-item {
   margin-left: 3em;
}

.editor-closer {
   margin-left: -0.5em;
}

.editor-properties-standard .uk-input[name=datePublished] {
   width: 20em;
}

.uk-iconnav .uk-button-small {
   line-height: 20px;
}

.editor-part-editor-toolbar {
   border-bottom: 1px solid #e5e5e5;
   padding-bottom: 5px;
   margin-bottom: 1em;
}

.editor-part-editor-iframe {
   height: 100%;
}

main.editor-html-editor {
   margin-top: 0em;
}

.editor-part-editor-source {
   height: 100%;
}

.editor-content-menu {
   float: left;
   padding-left: 0.25em;
   padding-top: 0.25em;
   width: 1.5em;
}

#editor-content-genre-list {
   margin-left: 2em;
}

#editor-content-genre-list .uk-accordion-title {
   border-bottom: 1px solid #e5e5e5;
}
#editor-content-genre-list .content-row .toolbar {
   min-width: 60px;
   padding-left: 0px;
   padding-right: 0px;
}
#editor-content-genre-list .content-row .date {
   min-width: 6em;
}
#editor-content-genre-list .content-row .time {
   min-width: 8em;
}
#editor-content-genre-list .content-row .headline {
   min-width: 10em;
}

#editor-login-dialog .uk-modal-dialog {
   height: 280px;
   width: 430px;
}
#editor-login-dialog-contents iframe {
   width: 100%;
   height: 100%;
}
