@media screen and (max-device-width: 480px) {

body {
   font-size: 18pt;
   border: 1px solid black;
}

main {
   flex-direction: column;
}

main > article {
   margin-right: 0px;
   min-width: auto;
   width: auto;
}

main > aside {
   float: none;
   width: auto;
   min-width: auto;
   margin-top: 2em;
   margin-left: 0em;
   margin-right: 0.5em;
   margin-bottom: 0.5em;
   padding-right: 0em;
   border: none;
}

main > aside > section {
   margin-bottom: 0em;
}

}
