
from duckpond.apps.editor import app
from duckpond.apps.editor.assets import *
from duckpond.apps.editor.views import *
from duckpond.apps.editor.api import *
from duckpond.apps.editor.model import *

if __name__ == '__main__':
    app.run()
