@media print {

@page {
   margin: 1cm 1cm 1cm 1cm;
}

body {
   margin: 0px;
   padding: 0px;
   background-color: white;
   font-size: 10pt !important;
}


aside { display: none}
article {
   margin: 1em;
   float:none;
   width: auto;
}

header h1 {
   float: none;
}

a:link, a:visited {
   color: inherit;
}

header .controls { display: none;}

header h1 { display: none; }

footer { display: none; }

h1 {
   color: black !important;
}

}
